@@ -1,16 +1,17 @@
-plot.linkdat<-function(x, marker=NULL, alleles=NULL, sep="", missing="-", margins = c(4.1, 1, 4.1, 1), ...) {
+plot.linkdat <- function(x, marker=NULL, alleles=NULL, sep="", missing="-", id.labels=x$orig.ids, title.name=x$famid, margins=c(4.1, 1, 4.1, 1), ...) {
 
-  ped <- x[['pedigree']];  AFF <- ped[,'AFF']; ID <- ped[,'ID']; SEX <- ped[,'SEX']
+  ped <- x$pedigree;  AFF <- ped[,'AFF']; ID <- ped[,'ID']; SEX <- ped[,'SEX']
   cols <- ifelse(AFF==0,8,1)  #unknown affection tegnes gr
   AFF[AFF==0] <- 2 #m gjre unknowns til affected for  unng bug
 
-  strid <- paste("", ID, sep="\n")
+  strid <- paste("", id.labels, sep="\n")
 
   if (!is.null(x$sim)) strid <- paste(strid, ifelse(x$sim==2, "*", ""), sep="")
 
   if (!is.null(marker)) {
-	if (is.null(m <- x[['markerdata']]) || marker > x[['nMark']]) stop("Indicated marker does not exist")
-	if (is.null(alleles)) alleles=attr(m, "alleles")
+	if (is.null(m <- x$markerdata) || marker > x$nMark) stop("Indicated marker does not exist")
+	if (is.null(alleles)) alleles=attr(m, "alleles")[marker]
+	else if(!is.list(alleles)) alleles=list(alleles)
 	chrom = ifelse(is.null(x$model), 'AUTOSOMAL', x$model$chrom)
 	geno = .prettyMarkers(m[, c(2*marker-1, 2*marker)], alleles=alleles, sep=sep, missing=missing, singleCol=TRUE, chrom=chrom, sex=SEX)
 	strid <- paste("", strid, geno, sep="\n")
@@ -18,8 +19,11 @@ plot.linkdat<-function(x, marker=NULL, alleles=NULL, sep="", missing="-", margin
   kped<-pedigree(id=ID, dadid=ped[,'FID'], momid=ped[,'MID'], sex=SEX, affected=AFF)
   par(xpd=T)
   plot.pedigree(kped, id=strid, col=cols, mar=margins, ...)
+  title(paste("Family", title.name))
 }
 
+
+
 .haploplot <- function(x, haplotypes, id=FALSE, cex=1, margins=c(4.1,1,4.1,1), ...) {
   stopifnot(class(x)=="linkdat", is.matrix(haplotypes), (ncol(haplotypes)%%2)==0)
   
